@@ -24,8 +24,9 @@
 #include "agentmanager.h"
 #include "controlmanager.h"
 #include "processcontrol.h"
+#include "akcrash.h"
+#include "akdebug.h"
 
-#include "kcrash.h"
 #include <stdlib.h>
 #include <config-akonadi.h>
 #ifdef HAVE_UNISTD_H
@@ -44,8 +45,8 @@ void crashHandler( int )
 
 int main( int argc, char **argv )
 {
-  KCrash::init();
-  KCrash::setEmergencyMethod( crashHandler );
+  akInit( QString::fromLatin1( "akonadi_control" ) );
+  AkonadiCrash::setEmergencyMethod( crashHandler );
 
   QCoreApplication app( argc, argv );
 
