@@ -22,15 +22,18 @@
 #include "akonadi.h"
 #include "akonadiconnection.h"
 #include "serveradaptor.h"
+#include "akdebug.h"
 
 #include "cachecleaner.h"
 #include "intervalcheck.h"
+#include "storage/dbconfig.h"
 #include "storage/datastore.h"
 #include "notificationmanager.h"
 #include "resourcemanager.h"
 #include "tracer.h"
 #include "xesammanager.h"
 #include "nepomukmanager.h"
+#include "debuginterface.h"
 
 #include <QtCore/QCoreApplication>
 #include <QtCore/QDir>
@@ -55,22 +58,21 @@ AkonadiServer::AkonadiServer( QObject* parent )
     , mDatabaseProcess( 0 )
     , mAlreadyShutdown( false )
 {
-    QSettings settings( XdgBaseDirs::akonadiServerConfigFile(), QSettings::IniFormat );
-    if ( settings.value( QLatin1String("General/Driver"), QLatin1String( "QMYSQL" ) ).toString() == QLatin1String( "QMYSQL" )
-         && settings.value( QLatin1String( "QMYSQL/StartServer" ), true ).toBool() )
-      startDatabaseProcess();
+    DbConfig::init();
+    startDatabaseProcess();
 
     s_instance = this;
 
-    const QString connectionSettingsFile = XdgBaseDirs::akonadiConnectionConfigFile(
-    XdgBaseDirs::WriteOnly );
+    const QString serverConfigFile = XdgBaseDirs::akonadiServerConfigFile( XdgBaseDirs::ReadWrite );
+    QSettings settings( serverConfigFile, QSettings::IniFormat );
 
+    const QString connectionSettingsFile = XdgBaseDirs::akonadiConnectionConfigFile( XdgBaseDirs::WriteOnly );
     QSettings connectionSettings( connectionSettingsFile, QSettings::IniFormat );
 
 #ifdef Q_OS_WIN
     QString namedPipe = settings.value( QLatin1String( "Connection/NamedPipe" ), QLatin1String( "Akonadi" ) ).toString();
     if ( !listen( namedPipe ) )
-      qFatal( "Unable to listen on Named Pipe %s", namedPipe.toLocal8Bit().data() );
+      akFatal() << "Unable to listen on Named Pipe" << namedPipe;
 
     connectionSettings.setValue( QLatin1String( "Data/Method" ), QLatin1String( "NamedPipe" ) );
     connectionSettings.setValue( QLatin1String( "Data/NamedPipe" ), namedPipe );
@@ -85,7 +87,7 @@ AkonadiServer::AkonadiServer( QObject* parent )
     const QString socketFile = socketDir + QLatin1String( "/akonadiserver.socket" );
     unlink( socketFile.toUtf8().constData() );
     if ( !listen( socketFile ) )
-      qFatal("Unable to listen on Unix socket '%s'", socketFile.toLocal8Bit().data() );
+      akFatal() << "Unable to listen on Unix socket" << socketFile;
 
     connectionSettings.setValue( QLatin1String( "Data/Method" ), QLatin1String( "UnixPath" ) );
     connectionSettings.setValue( QLatin1String( "Data/UnixPath" ), socketFile );
@@ -94,12 +96,13 @@ AkonadiServer::AkonadiServer( QObject* parent )
     // initialize the database
     DataStore *db = DataStore::self();
     if ( !db->database().isOpen() )
-      qFatal("Unable to open database.");
+      akFatal() << "Unable to open database" << db->database().lastError().text();
     if ( !db->init() )
-      qFatal("Unable to initialize database.");
+      akFatal() << "Unable to initialize database.";
 
     NotificationManager::self();
     Tracer::self();
+    new DebugInterface( this );
     ResourceManager::self();
     if ( settings.value( QLatin1String( "Cache/EnableCleaner" ), true ).toBool() ) {
       mCacheCleaner = new CacheCleaner( this );
@@ -114,8 +117,8 @@ AkonadiServer::AkonadiServer( QObject* parent )
     new ServerAdaptor( this );
     QDBusConnection::sessionBus().registerObject( QLatin1String( "/Server" ), this );
 
-    char* dbusAddress = getenv( "DBUS_SESSION_BUS_ADDRESS" );
-    if ( dbusAddress != 0 ) {
+    const QByteArray dbusAddress = qgetenv( "DBUS_SESSION_BUS_ADDRESS" );
+    if ( !dbusAddress.isEmpty() ) {
       connectionSettings.setValue( QLatin1String( "DBUS/Address" ), QLatin1String( dbusAddress ) );
     }
 }
@@ -158,8 +161,7 @@ void AkonadiServer::quit()
     // and the following db termination will work
     QCoreApplication::instance()->processEvents();
 
-    if ( mDatabaseProcess )
-      stopDatabaseProcess();
+    stopDatabaseProcess();
 
     QSettings settings( XdgBaseDirs::akonadiServerConfigFile(), QSettings::IniFormat );
     const QString connectionSettingsFile = XdgBaseDirs::akonadiConnectionConfigFile( XdgBaseDirs::WriteOnly );
@@ -170,10 +172,10 @@ void AkonadiServer::quit()
     const QString socketDir = settings.value( QLatin1String( "Connection/SocketDirectory" ), defaultSocketDir ).toString();
 
     if ( !QDir::home().remove( socketDir + QLatin1String( "/akonadiserver.socket" ) ) )
-        qWarning("Failed to remove Unix socket");
+        akError() << "Failed to remove Unix socket";
 #endif
     if ( !QDir::home().remove( connectionSettingsFile ) )
-        qWarning("Failed to remove runtime connection config file");
+        akError() << "Failed to remove runtime connection config file";
 
     QTimer::singleShot( 0, this, SLOT( doQuit() ) );
 }
@@ -201,13 +203,12 @@ AkonadiServer * AkonadiServer::instance()
 
 void AkonadiServer::startDatabaseProcess()
 {
-#ifdef MYSQLD_EXECUTABLE
-  const QString mysqldPath = QLatin1String( MYSQLD_EXECUTABLE );
-#else
-  const QString mysqldPath;
-  Q_ASSERT_X( false, "AkonadiServer::startDatabaseProcess()",
-              "mysqld was not found during compile time, you need to start a MySQL server yourself first and configure Akonadi accordingly" );
-#endif
+  if ( !DbConfig::useInternalServer() )
+    return;
+
+  const QString mysqldPath = DbConfig::serverPath();
+  if ( mysqldPath.isEmpty() )
+    akFatal() << "No path to mysqld set in server configuration!";
 
   // create the database directories if they don't exists
   const QString dataDir = XdgBaseDirs::saveDir( "data", QLatin1String( "akonadi/db_data" ) );
@@ -219,7 +220,7 @@ void AkonadiServer::startDatabaseProcess()
   const QString localConfig  = XdgBaseDirs::findResourceFile( "config", QLatin1String( "akonadi/mysql-local.conf" ) );
   const QString actualConfig = XdgBaseDirs::saveDir( "data", QLatin1String( "akonadi" ) ) + QLatin1String("/mysql.conf");
   if ( globalConfig.isEmpty() )
-    qFatal("Where is my MySQL config file??");
+    akFatal() << "Did not find MySQL server default configuration (mysql-global.conf)";
   QFile globalFile( globalConfig );
   QFile actualFile( actualConfig );
   if ( globalFile.open( QFile::ReadOnly ) && actualFile.open( QFile::WriteOnly ) ) {
@@ -234,17 +235,34 @@ void AkonadiServer::startDatabaseProcess()
     actualFile.close();
     globalFile.close();
   } else {
-    qFatal("What did you do to my MySQL config file??");
+    akError() << "Unable to create MySQL server configuration file.";
+    akError() << "This means that either the default configuration file (mysql-global.conf) was not readable";
+    akFatal() << "or the target file (mysql.conf) could not be written.";
   }
 
   if ( dataDir.isEmpty() )
-    qFatal("Akonadi server was not able not create database data directory");
+    akFatal() << "Akonadi server was not able not create database data directory";
 
   if ( akDir.isEmpty() )
-    qFatal("Akonadi server was not able not create database log directory");
+    akFatal() << "Akonadi server was not able not create database log directory";
 
   if ( miscDir.isEmpty() )
-    qFatal("Akonadi server was not able not create database misc directory");
+    akFatal() << "Akonadi server was not able not create database misc directory";
+
+  // move mysql error log file out of the way
+  const QFileInfo errorLog( dataDir + QDir::separator() + QString::fromLatin1( "mysql.err" ) );
+  if ( errorLog.exists() ) {
+    QFile logFile( errorLog.absoluteFilePath() );
+    QFile oldLogFile( dataDir + QDir::separator() + QString::fromLatin1( "mysql.err.old" ) );
+    if ( logFile.open( QFile::ReadOnly ) && oldLogFile.open( QFile::Append ) ) {
+      oldLogFile.write( logFile.readAll() );
+      oldLogFile.close();
+      logFile.close();
+      logFile.remove();
+    } else {
+      akError() << "Failed to open MySQL error log.";
+    }
+  }
 
   // synthesize the mysqld command
   QStringList arguments;
@@ -254,36 +272,61 @@ void AkonadiServer::startDatabaseProcess()
 
   mDatabaseProcess = new QProcess( this );
   mDatabaseProcess->start( mysqldPath, arguments );
-  if ( !mDatabaseProcess->waitForStarted() )
-    qFatal( "Could not start database server '%s'", qPrintable( mysqldPath ) );
+  if ( !mDatabaseProcess->waitForStarted() ) {
+    akError() << "Could not start database server!";
+    akError() << "executable:" << mysqldPath;
+    akError() << "arguments:" << arguments;
+    akFatal() << "process error:" << mDatabaseProcess->errorString();
+  }
 
+  const QLatin1String initCon( "initConnection" );
   {
-    QSqlDatabase db = QSqlDatabase::addDatabase( QLatin1String( "QMYSQL" ), QLatin1String( "initConnection" ) );
-    db.setConnectOptions( QString::fromLatin1( "UNIX_SOCKET=%1/mysql.socket" ).arg( miscDir ) );
+    QSqlDatabase db = QSqlDatabase::addDatabase( DbConfig::driverName(), initCon );
+    DbConfig::configure( db );
+    db.setDatabaseName( QString() ); // might not exist yet, then connecting to the actual db will fail
+    if ( !db.isValid() )
+      akFatal() << "Invalid database object during database server startup";
 
     bool opened = false;
     for ( int i = 0; i < 120; ++i ) {
       opened = db.open();
       if ( opened )
         break;
-      if ( mDatabaseProcess->waitForFinished( 500 ) )
-        qFatal( "Database server exited unexpectedly, exit code %d", mDatabaseProcess->exitCode() );
+      if ( mDatabaseProcess->waitForFinished( 500 ) ) {
+        akError() << "Database process existed unexpectedly during intial connection!";
+        akError() << "executable:" << mysqldPath;
+        akError() << "arguments:" << arguments;
+        akError() << "stdout:" << mDatabaseProcess->readAllStandardOutput();
+        akError() << "stderr:" << mDatabaseProcess->readAllStandardError();
+        akError() << "exit code:" << mDatabaseProcess->exitCode();
+        akFatal() << "process error:" << mDatabaseProcess->errorString();
+      }
     }
 
     if ( opened ) {
       QSqlQuery query( db );
-      if ( !query.exec( QLatin1String( "USE DATABASE akonadi" ) ) )
-        query.exec( QLatin1String( "CREATE DATABASE akonadi" ) );
-
+      if ( !query.exec( QString::fromLatin1( "USE %1" ).arg( DbConfig::databaseName() ) ) ) {
+        akDebug() << "Failed to use database" << DbConfig::databaseName();
+        akDebug() << "Query error:" << query.lastError().text();
+        akDebug() << "Database error:" << db.lastError().text();
+        akDebug() << "Trying to create database now...";
+        if ( !query.exec( QLatin1String( "CREATE DATABASE akonadi" ) ) ) {
+          akError() << "Failed to create database";
+          akError() << "Query error:" << query.lastError().text();
+          akFatal() << "Database error:" << db.lastError().text();
+        }
+      }
       db.close();
     }
   }
 
-  QSqlDatabase::removeDatabase( QLatin1String( "initConnection" ) );
+  QSqlDatabase::removeDatabase( initCon );
 }
 
 void AkonadiServer::stopDatabaseProcess()
 {
+  if ( !mDatabaseProcess )
+    return;
   mDatabaseProcess->terminate();
   mDatabaseProcess->waitForFinished();
 }
