@@ -44,7 +44,6 @@ void IntervalCheck::run()
 
 void IntervalCheck::doIntervalCheck()
 {
-  qDebug() << "doing interval checks...";
 
   // cycle over all locations
   QList<Location> locations = Location::retrieveAll();
@@ -65,8 +64,6 @@ void IntervalCheck::doIntervalCheck()
     DataStore::self()->triggerCollectionSync( location );
   }
 
-  qDebug() << "interval checks done";
-
   QTimer::singleShot( 60 * 1000, this, SLOT(doIntervalCheck()) );
 }
 
